from __future__ import print_function
import sys

import PyQt4.QtCore as QtCore
import PyQt4.QtScript as QtScript
import PyQt4.QtGui as QtGui

app = QtGui.QApplication(sys.argv)

from twisted.application import reactors
reactors.installReactor('qt4')

from twisted.internet import reactor, task
from twisted.python import log

log.startLogging(sys.stdout)


class DoNothing(QtCore.QObject):
    def __init__(self):
        self.count = 0
        self.looping = False
        task.LoopingCall(self.print_stat).start(1.0)
        QtCore.QObject.__init__(self)

    def do_something(self):
        if not self.looping:
            return

        self.count += 1
        reactor.callLater(0.003, self.do_something)

    def button_click(self):
        if self.looping:
            self.looping = False
            log.msg('looping stopped....')
        else:
            self.looping = True
            self.do_something()
            log.msg('looping started....')

    def print_stat(self):
        log.msg(' c: ' + str(self.count))


t = DoNothing()

engine = QtScript.QScriptEngine()

button = QtGui.QPushButton()
scriptButton = engine.newQObject(button)
engine.globalObject().setProperty("button", scriptButton)

app.connect(button, QtCore.SIGNAL("clicked()"), t.button_click)

engine.evaluate("button.text = 'Hello World!'")
engine.evaluate("button.styleSheet = 'font-style: italic'")
engine.evaluate("button.show()")

# noinspection PyUnresolvedReferences
reactor.runReturn()
app.exec_()
log.msg('fell off the bottom?...')





from __future__ import print_function

__author__ = 'ght'

import sys

from twisted.application import reactors

reactors.installReactor('qt4')

from twisted.internet import reactor
from twisted.python import log

log.startLogging(sys.stdout)

resolve_helper = """
import sys
print sys.path
import {0}
{0}.install()
from twisted.internet import reactor

class Foo:
    def __init__(self):
        print sys.path
        reactor.callWhenRunning(self.start)
        self.timer = reactor.callLater(3, self.failed)
    def start(self):
        reactor.resolve('localhost').addBoth(self.done)
    def done(self, res):
        print 'done', res
        reactor.stop()
    def failed(self):
        print 'failed'
        self.timer = None
        reactor.stop()
f = Foo()
reactor.run()
""".format

import os

from twisted.internet import protocol, defer


class ChildResolveProtocol(protocol.ProcessProtocol):
    def __init__(self, onCompletion):
        self.onCompletion = onCompletion

    def connectionMade(self):
        self.output = []
        self.error = []

    def outReceived(self, out):
        self.output.append(out)

    def errReceived(self, err):
        self.error.append(err)

    def processEnded(self, reason):
        self.onCompletion.callback((reason, self.output, self.error))
        self.onCompletion = None

def spawn():

    helperPath = '/tmp/testspawn.py'
    helperFile = open(helperPath, 'w')

    # Eeueuuggg
    reactorName = reactor.__module__

    s=resolve_helper(reactorName)

    helperFile.write(s)
    helperFile.close()

    env = os.environ.copy()
    s=os.pathsep.join(['/Users/ght/PycharmProjects/qtreactor']+sys.path)
    log.msg("My Beautiful PATH",s," END OF MY BEAUTIFUL PATH")
    env['PYTHONPATH'] = s

    log.msg(env)

    helperDeferred = defer.Deferred()
    helperProto = ChildResolveProtocol(helperDeferred)

    reactor.spawnProcess(helperProto, sys.executable, ("python", "-u", helperPath), env)

    return helperDeferred

d = spawn()

def whathapp(result):
    log.msg("finished ", result)
    reactor.stop()

d.addCallback(whathapp)

reactor.run()
log.msg('fell off the bottom?...')



from __future__ import print_function

import sys

from PySide import QtGui, QtScript
from PySide.QtCore import SIGNAL

app = QtGui.QApplication(sys.argv)

from qtreactor import pyqt4reactor

pyqt4reactor.install()

from twisted.internet import reactor, task
from twisted.python import log

log.startLogging(sys.stdout)


def test_reactor():
    print('tick...')


def button_click():
    print('click...')
    reactor.iterate(5.0)
    print('click return')


engine = QtScript.QScriptEngine()

button = QtGui.QPushButton()
scriptButton = engine.newQObject(button)
engine.globalObject().setProperty("button", scriptButton)

app.connect(button, SIGNAL("clicked()"), button_click)

engine.evaluate("button.text = 'Hello World!'")
engine.evaluate("button.styleSheet = 'font-style: italic'")
engine.evaluate("button.show()")

task.LoopingCall(test_reactor).start(1.0)
reactor.run()
log.msg('fell off the bottom?...')



from __future__ import print_function

from PyQt4 import QtCore
from PyQt4.QtGui import *

import sys
app = QApplication(sys.argv)

import qtreactor.pyqt4reactor
qtreactor.pyqt4reactor.install()

#reactors.installReactor('pyqt4')

from twisted.internet import reactor, task

class AppForm(QMainWindow):
    def __init__(self, parent=None):
        QMainWindow.__init__(self, parent)
        self.qpingcount=0
        self.create_main_frame()

    def create_main_frame(self):        
        page = QWidget()        

        self.button = QPushButton('joy', page)
        #self.button2 = QPushButton('loop', page)
        self.edit1 = QLineEdit()
        self.edit2 = QLineEdit()

        vbox1 = QVBoxLayout()
        #vbox1.addWidget(self.button2)
        vbox1.addWidget(self.edit1)
        vbox1.addWidget(self.edit2)
        vbox1.addWidget(self.button)
        page.setLayout(vbox1)
        self.setCentralWidget(page)

        self.connect(self.button,
                     QtCore.SIGNAL("clicked()"),
                     self.clicked2)

        #self.connect(self.button2,
        #             QtCore.SIGNAL("clicked()"),
        #             self.spawnclick)

        self._timer = QtCore.QTimer()
        self._timer.setSingleShot(False)
        QtCore.QObject.connect(self._timer,
                               QtCore.SIGNAL("timeout()"),
                               self.qclick)

        self._timer.setInterval(1000)
        self._timer.start()


    def spawnclick(self):
        self.lc = task.LoopingCall(self.pingbox)
        self.lc.start(1.0)

    def clicked1(self):
        # todo, the inspector isn't happy here either
        QMessageBox.about(self, "My message box", "Text1 = %s, Text2 = %s" % (
            self.edit1.text(), self.edit2.text()))

    def qclick(self):
        self.qpingcount += 1
        self.edit1.setText(str(self.qpingcount))

    def clicked2(self):
        self.edit1.setText("die mutherfucker")
        try:
            self.lc.stop()
        except:
            pass
        finally:
            reactor.callLater(1.0, reactor.stop)

    def pingbox(self):
        self.edit2.setText(str(qtreactor.pyqt4reactor.pyqt4reactor.pingcount))


lc = None
form = None

def buildgui():
    form = AppForm()
    form.show()
    return form

if __name__ == "__main__":
    form = buildgui()
    #app.exec_()
    reactor.run()





from __future__ import print_function
__author__ = 'ght'

# Copyright (c) Twisted Matrix Laboratories.
# See LICENSE for details.

from twisted.application import reactors

reactors.installReactor('qt4')

from twisted.python import log
import sys
log.startLogging(sys.stdout)

"""
An example client. Run simpleserv.py first before running this.
"""

from twisted.internet import reactor, protocol

import q

# a client protocol

class EchoClient(protocol.Protocol):
    """Once connected, send a message, then print the result."""

    def connectionMade(self):
        q('client')
        self.transport.write("hello, world!")

    def dataReceived(self, data):
        q('client received: ', data)
        "As soon as any data is received, write it back."
        log.msg("Server said:", data)
        reactor.callLater(1.0, self.writeAgain)
        #self.transport.loseConnection()

    def writeAgain(self):
        q('client doing it again')
        self.transport.write("Helloooo again!!!!")

    def connectionLost(self, reason):
        q('client')
        log.msg("connection lost")

class EchoFactory(protocol.ClientFactory):
    protocol = EchoClient

    def clientConnectionFailed(self, connector, reason):
        q('client',connector,reason)
        log.msg("Connection failed - goodbye!")
        #reactor.stop()

    def clientConnectionLost(self, connector, reason):
        q('client',connector,reason)
        log.msg("Connection lost - goodbye!")
        #reactor.stop()


# this connects the protocol to a server runing on port 8000
def main():
    f = EchoFactory()
    reactor.connectTCP("127.0.0.1", 8000, f)
    reactor.run()

# this only runs if the module was *not* imported
if __name__ == '__main__':
    main()


# Copyright (c) 2001-2011 Twisted Matrix Laboratories.
# See LICENSE for details.
"""
See main docstring in module __init__.py

Reactor supporting PyQt4

API Stability: stable

Maintainer: U{Glenn H Tarbox, PhD<mailto:glenn@tarbox.org>}

"""

from __future__ import print_function, absolute_import

from twisted.python import runtime

from qtreactor import qtreactor_config

qtreactor_config.set_qt_name("PySide")

from qtreactor import qt4base

class pyqt4reactor(qt4base.QtReactor):
    pass

class pyqt4eventreactor(qt4base.QtReactor):
    pass

def posixinstall():
    """
    Install the Qt reactor.
    """
    p = pyqt4reactor()
    from twisted.internet.main import installReactor

    installReactor(p)


def win32install():
    """
    Install the Qt reactor.
    """
    p = pyqt4eventreactor()
    from twisted.internet.main import installReactor

    installReactor(p)

if runtime.platform.getType() == 'win32':
    install = win32install
else:
    install = posixinstall

__all__ = ["install"]



__author__ = 'ght'

qt_preset = "PyQt4"

class _config(object):
    def __init__(self):
        self.qtname = qt_preset
        if self.qtname != "":
            self.preset = True

_instance = _config()

def set_qt_name(name):
    if _instance.preset == True and name != _instance.qtname:
        raise Exception("Qt Static Configuration")

    _instance.qtname = name

def get_qt_name():
    return _instance.qtname



from __future__ import print_function

__author__ = 'ght'

# Copyright (c) Twisted Matrix Laboratories.
# See LICENSE for details.

# ---------------------------------
# Client

from twisted.application import reactors

reactors.installReactor('pyqt4')

from twisted.trial import unittest
from twisted.internet.protocol import DatagramProtocol
from twisted.internet import reactor, task


# noinspection PyClassHasNoInit
class EchoClientDatagramProtocol(DatagramProtocol):
    long_string = "-" * 1000

    def __init__(self):
        self.sendflag = True

    def stop_sending(self):
        self.sendflag = False

    def startProtocol(self):
        self.transport.connect('127.0.0.1', self.useport)
        self.senddatagram()

    def senddatagram(self):
        datagram = self.long_string
        self.transport.write(datagram)

    def datagramReceived(self, datagram, host):
        if self.sendflag:
            reactor.callLater(0.001, self.senddatagram)


def build_client(port, stopsend, stoplisten):
    protocol = EchoClientDatagramProtocol()
    protocol.useport = port
    r = reactor.listenUDP(0, protocol)
    reactor.callLater(stoplisten, r.stopListening)
    reactor.callLater(stopsend, protocol.stop_sending)
    return protocol

#------------------------------------------
# Server

# noinspection PyClassHasNoInit
class EchoUDP(DatagramProtocol):
    def datagramReceived(self, datagram, address):
        #log.msg("server received: ", repr(datagram), repr(address))
        self.transport.write(datagram, address)

def build_server(port, timeout):
    echo = EchoUDP()
    r = reactor.listenUDP(port, echo)
    reactor.callLater(timeout, r.stopListening)
    return echo

class TrialTest(unittest.TestCase):

    def setup(self):
        print("setup")

    def test_main(self):
        begin_port = 5010
        for port in xrange(begin_port, begin_port + 50):
            build_client(port, 10, 11)
            build_server(port, 12)

        def callme():
            print("deferLater callback")
            return 'he called'

        return task.deferLater(reactor, 15, callme)

    def tearDown(self):
        print('teardown')



from __future__ import print_function

import sys

from twisted.application import reactors

reactors.installReactor('qt4')

from twisted.internet import reactor, task
from twisted.python import log, runtime

log.startLogging(sys.stdout)

def testreactor():
    print('tick...')


def doit():
    log.msg("reactor module: ", reactor.__module__)
    task.LoopingCall(testreactor).start(1.0)
    reactor.callLater(5.0, reactor.stop)
    log.msg("platform runtime: " + repr(runtime.platform.getType()))


reactor.callWhenRunning(doit)
log.msg('calling reactor.run()')
reactor.run()
log.msg('fell off the bottom?...')



from __future__ import print_function, absolute_import

import sys

import PyQt4.QtGui as QtGui
import PyQt4.QtScript as QtScript
from PyQt4.QtCore import SIGNAL

app = QtGui.QApplication(sys.argv)

from qtreactor import pyqt4reactor

pyqt4reactor.install()


class DoNothing(object):
    def __init__(self):
        self.count = 0
        self.running = False

    def button_click(self):
        if not self.running:
            from twisted.scripts import trial
            trial.run()


def run():

    t = DoNothing()

    engine = QtScript.QScriptEngine()

    button = QtGui.QPushButton()
    button = engine.newQObject(button)
    engine.globalObject().setProperty("button", button)

    app.connect(button, SIGNAL("clicked()"), t.button_click)

    engine.evaluate("button.text = 'Do Twisted Gui Trial'")
    engine.evaluate("button.styleSheet = 'font-style: italic'")
    engine.evaluate("button.show()")

    app.exec_()
    print('fell off the bottom?...')



from __future__ import print_function
__author__ = 'ght'

import qtreactor.pyqt4reactor
#qtreactor.pyqt4reactor.install()

from twisted.application import reactors
reactors.installReactor('pyqt4')

#from ghtTests.texboxtest import buildgui

#form = buildgui()

import sys
#log.startLogging(sys.stdout)

trialpath = '/usr/local/bin/trial'
trial = open(trialpath,'r').read()

import contextlib
@contextlib.contextmanager
def redirect_argv(num):
    sys._argv = sys.argv[:]
    sys.argv[:] = num
    yield
    sys.argv[:] = sys._argv

def runTrial(*trials):
    with redirect_argv([trialpath]+list(trials)):
        exec (trial)

if __name__ == '__main__':
    runTrial(*['twisted.test.test_ftp',
              'twisted.test.test_internet'])



__author__ = 'ght'


from twisted.application import reactors

reactors.installReactor('kqueue')

# Copyright (c) Twisted Matrix Laboratories.
# See LICENSE for details.

"""
Test running processes.
"""

import gzip
import os
import sys
import signal
import StringIO
import errno
import gc
import stat
import operator
try:
    import fcntl
except ImportError:
    fcntl = process = None
else:
    from twisted.internet import process


from zope.interface.verify import verifyObject

from twisted.python.log import msg
from twisted.internet import reactor, protocol, error, interfaces, defer
from twisted.trial import unittest
from twisted.python import util, runtime, procutils

from twisted.test import test_process

#testroot = "/usr/local/lib/python2.7/site-packages/twisted/test/process_fds.py"

testroot = __file__

class FDChecker(protocol.ProcessProtocol):
    state = 0
    data = ""
    failed = None

    def __init__(self, d):
        self.deferred = d

    def fail(self, why):
        self.failed = why
        self.deferred.callback(None)

    def connectionMade(self):
        self.transport.writeToChild(0, "abcd")
        self.state = 1

    def childDataReceived(self, childFD, data):
        if self.state == 1:
            if childFD != 1:
                self.fail("read '%s' on fd %d (not 1) during state 1" \
                          % (childFD, data))
                return
            self.data += data
            #print "len", len(self.data)
            if len(self.data) == 6:
                if self.data != "righto":
                    self.fail("got '%s' on fd1, expected 'righto'" \
                              % self.data)
                    return
                self.data = ""
                self.state = 2
                #print "state2", self.state
                self.transport.writeToChild(3, "efgh")
                return
        if self.state == 2:
            self.fail("read '%s' on fd %s during state 2" % (childFD, data))
            return
        if self.state == 3:
            if childFD != 1:
                self.fail("read '%s' on fd %s (not 1) during state 3" \
                          % (childFD, data))
                return
            self.data += data
            if len(self.data) == 6:
                if self.data != "closed":
                    self.fail("got '%s' on fd1, expected 'closed'" \
                              % self.data)
                    return
                self.state = 4
            return
        if self.state == 4:
            self.fail("read '%s' on fd %s during state 4" % (childFD, data))
            return

    def childConnectionLost(self, childFD):
        if self.state == 1:
            self.fail("got connectionLost(%d) during state 1" % childFD)
            return
        if self.state == 2:
            if childFD != 4:
                self.fail("got connectionLost(%d) (not 4) during state 2" \
                          % childFD)
                return
            self.state = 3
            self.transport.closeChildFD(5)
            return

    def processEnded(self, status):
        rc = status.value.exitCode
        if self.state != 4:
            self.fail("processEnded early, rc %d" % rc)
            return
        if status.value.signal != None:
            self.fail("processEnded with signal %s" % status.value.signal)
            return
        if rc != 0:
            self.fail("processEnded with rc %d" % rc)
            return
        self.deferred.callback(None)


class FDTest(unittest.TestCase):

    def testFD(self):
        exe = sys.executable
        scriptPath = util.sibpath(testroot, "process_fds.py")
        d = defer.Deferred()
        p = FDChecker(d)
        reactor.spawnProcess(p, exe, [exe, "-u", scriptPath], env=None,
                             path=None,
                             childFDs={0:"w", 1:"r", 2:2,
                                       3:"w", 4:"r", 5:"w"})
        def processEnded(ign):
            return self.failIf(p.failed, p.failed)
        d.addCallback(processEnded)
        return d

    def testLinger(self):
        # See what happens when all the pipes close before the process
        # actually stops. This test *requires* SIGCHLD catching to work,
        # as there is no other way to find out the process is done.
        exe = sys.executable
        scriptPath = util.sibpath(testroot, "process_linger.py")
        p = Accumulator()
        d = p.endedDeferred = defer.Deferred()
        reactor.spawnProcess(p, exe, [exe, "-u", scriptPath], env=None,
                             path=None,
                             childFDs={1:"r", 2:2},
                             )
        def processEnded(ign):
            self.assertEqual(p.outF.getvalue(),
                                 "here is some text\ngoodbye\n")
        return d.addCallback(processEnded)


class Accumulator(protocol.ProcessProtocol):
    """Accumulate data from a process."""

    closed = 0
    endedDeferred = None

    def connectionMade(self):
        self.outF = StringIO.StringIO()
        self.errF = StringIO.StringIO()

    def outReceived(self, d):
        self.outF.write(d)

    def errReceived(self, d):
        self.errF.write(d)

    def outConnectionLost(self):
        pass

    def errConnectionLost(self):
        pass

    def processEnded(self, reason):
        self.closed = 1
        if self.endedDeferred is not None:
            d, self.endedDeferred = self.endedDeferred, None
            d.callback(None)



from __future__ import print_function

import sys

from PyQt4 import QtGui, QtScript
from PyQt4 import QtCore

app = QtGui.QApplication(sys.argv)

from qtreactor import pyqt4reactor

pyqt4reactor.install()

from twisted.internet import reactor, task


class DoNothing(object):
    def __init__(self):
        self.count = 0
        self.running = False
        task.LoopingCall(self.print_stat).start(1.0)

    def button_click(self):
        if self.running:
            self.running = False
            print('CLICK: calling reactor stop...')
            reactor.stop()
            print('reactor stop called....')
        else:
            self.running = True
            print('CLICK: entering run')
            reactor.run()
            print('reactor run returned...')

    @staticmethod
    def print_stat():
        print('tick...')


t = DoNothing()

engine = QtScript.QScriptEngine()

button = QtGui.QPushButton()
scriptButton = engine.newQObject(button)
engine.globalObject().setProperty("button", scriptButton)

app.connect(button, QtCore.SIGNAL("clicked()"), t.button_click)

engine.evaluate("button.text = 'Hello World!'")
engine.evaluate("button.styleSheet = 'font-style: italic'")
engine.evaluate("button.show()")

app.exec_()
print('fell off the bottom?...')



from __future__ import print_function

from qtreactor import pyqt4reactor

from PyQt4 import QtGui

app = QtGui.QApplication(sys.argv)
pyqt4reactor.install()

from twisted.words.protocols import irc
from twisted.internet import reactor, protocol

import sys


# noinspection PyClassHasNoInit
class IRCCore(irc.IRCClient):
    nickname = 'dosdsdssd'

    def connectionMade(self):
        self.nickname = self.factory.window.nickName.text().encode('ascii')
        self.factory.window.protocol = self
        irc.IRCClient.connectionMade(self)
        self.log('connected!!')

    def connectionLost(self, reason):
        self.log('disconnected... :( %s' % reason)

    def signedOn(self):
        charname = self.factory.window.channelName.text().encode('ascii')
        self.join(charname)

    def joined(self, channel):
        self.log('joined %s' % channel)

    def privmsg(self, user, channel, msg):
        self.log('%s %s %s' % (user, channel, msg))

    def action(self, user, channel, msg):
        self.log('action: %s %s %s' % (user, channel, msg))

    def log(self, log_str):
        self.factory.window.view.addItem(log_str)


class IRCCoreFactory(protocol.ClientFactory):
    protocol = IRCCore

    def __init__(self, window):
        self.window = window

    def clientConnectionLost(self, connector, reason):
        # reconnect to server if lose connection
        connector.connect()

    def clientConnectionFailed(self, connector, reason):
        print('connection failed! :(', reason)
        reactor.stop()


class MainWindow(QtGui.QMainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()

        layout = QtGui.QHBoxLayout()

        layout.addWidget(QtGui.QLabel('Server:'))

        self.serverName = QtGui.QLineEdit('irc.freenode.org')

        layout.addWidget(self.serverName)

        layout.addWidget(QtGui.QLabel('Channel:'))

        self.channelName = QtGui.QLineEdit('#pangaea')

        layout.addWidget(self.channelName)

        layout.addWidget(QtGui.QLabel('Nick:'))

        self.nickName = QtGui.QLineEdit('ceruleanwave9832')

        layout.addWidget(self.nickName)

        self.connectButton = QtGui.QPushButton('Connect!')

        layout.addWidget(self.connectButton)

        self.connectButton.clicked.connect(self.connect_irc)

        self.view = QtGui.QListWidget()

        self.entry = QtGui.QLineEdit()

        self.entry.returnPressed.connect(self.send_message)

        irc_widget = QtGui.QWidget(self)

        vbox = QtGui.QVBoxLayout()

        vbox.addLayout(layout)

        vbox.addWidget(self.view)

        vbox.addWidget(self.entry)

        irc_widget.setLayout(vbox)

        self.setCentralWidget(irc_widget)

        self.setWindowTitle('IRC')

        self.setUnifiedTitleAndToolBarOnMac(True)

        self.showMaximized()

        self.protocol = None

    def connect_irc(self):
        self.connectButton.setDisabled(True)

        self.channelName.setDisabled(True)

        self.nickName.setDisabled(True)

        self.serverName.setDisabled(True)

        factory = IRCCoreFactory(self)

        server_name = self.serverName.text().encode('ascii')

        reactor.connectTCP(server_name, 6667, factory)

        # reactor.runReturn()
        # app.exit()
        #app.exit()

        reactor.run()

    def send_message(self):
        if self.protocol:
            chan_name = self.channelName.text().encode('ascii')

            message = self.entry.text().encode('ascii')

            self.protocol.msg(chan_name, message)

            self.view.addItem('%s <%s> %s' % (chan_name, self.protocol.nickname, message))
        else:
            self.view.addItem('Not connected.')
        self.entry.setText('')

    def closeEvent(self, event):
        print('Attempting to close the main window!')
        reactor.stop()
        event.accept()


if __name__ == '__main__':
    mainWin = MainWindow()
    sys.exit(app.exec_())



from __future__ import print_function
__author__ = 'ght'

from twisted.application import reactors

reactors.installReactor('qt4')

from twisted.trial import unittest
from twisted.internet import reactor, task


class TrialTest1(unittest.TestCase):

    def setUp(self):
        print("setUp()")

    def test_main(self):
        print("test_main")
        return task.deferLater(reactor, 1, self._called_by_deffered1)

    def _called_by_deffered1(self):
        print("_called_by_deffered1")
        return task.deferLater(reactor, 1, self._called_by_deffered2)

    def _called_by_deffered2(self):
        print("_called_by_deffered2")

    def tearDown(self):
        print("tearDown()")




# Copyright (c) 2001-2011 Twisted Matrix Laboratories.
# See LICENSE for details.
"""
This module provides support for Twisted to be driven by the Qt mainloop.

In order to use this support, simply do the following::
    |  app = QApplication(sys.argv) # your code to init Qt
    |  import qt4reactor
    |  qt4reactor.install()

alternatively:

    |  from twisted.application import reactors
    |  reactors.installReactor('qt4')

Then use twisted.internet APIs as usual.  The other methods here are not
intended to be called directly.

If you don't instantiate a QApplication or QCoreApplication prior to
installing the reactor, a QCoreApplication will be constructed
by the reactor.  QCoreApplication does not require a GUI so trial testing
can occur normally.

Twisted can be initialized after QApplication.exec_() with a call to
reactor.runReturn().  calling reactor.stop() will unhook twisted but
leave your Qt application running

API Stability: stable

Maintainer: U{Glenn H Tarbox, PhD<mailto:glenn@tarbox.org>}

Previous maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
Original port to QT4: U{Gabe Rudy<mailto:rudy@goldenhelix.com>}
Subsequent port by therve
"""

from __future__ import print_function, absolute_import

import sys

if __name__ == '__main__':
    from twisted.application import reactors

    reactors.installReactor('pyqt4')

from zope.interface import implements
from twisted.internet.interfaces import IReactorFDSet
from twisted.python import log, runtime
from twisted.internet import posixbase

from qtreactor import qtreactor_config

if qtreactor_config.get_qt_name() == "PyQt4":
    from PyQt4 import QtCore
elif qtreactor_config.get_qt_name() == "PySide":
    from PySide import QtCore
else:
    raise Exception("Must Have PyQt4 or PySide")


# noinspection PyBroadException,PyProtectedMember
class TwistedSocketNotifier(QtCore.QObject):
    """
    Connection between an fd event and reader/writer callbacks.
    """

    def __init__(self, parent, qt_reactor, watcher, socket_type):
        QtCore.QObject.__init__(self, parent)
        self.qt_reactor = qt_reactor
        self.watcher = watcher
        fd = watcher.fileno()
        self.notifier = QtCore.QSocketNotifier(fd, socket_type, parent)
        self.notifier.setEnabled(True)
        if socket_type == QtCore.QSocketNotifier.Read:
            self.fn = self.read
        else:
            self.fn = self.write
        QtCore.QObject.connect(self.notifier,
                               QtCore.SIGNAL("activated(int)"),
                               self.fn)

    def shutdown(self):
        self.notifier.setEnabled(False)
        self.disconnect(self.notifier, QtCore.SIGNAL("activated(int)"), self.fn)
        self.fn = self.watcher = None
        self.notifier.deleteLater()
        self.deleteLater()

    # noinspection PyUnusedLocal
    def read(self, fd):
        self.notifier.setEnabled(False)
        if not self.watcher:
            return
        w = self.watcher
        # doRead can cause self.shutdown to be called so keep a
        # reference to self.watcher

        def _read():
            try:
                data = w.doRead()
                if data:
                    self.qt_reactor._disconnectSelectable(w, data, True)
                elif self.watcher:
                    self.notifier.setEnabled(True)
            except:
                log.err()
                self.qt_reactor._disconnectSelectable(w, sys.exc_info()[1], False)
            self.qt_reactor._iterate(None, fromqt=True)

        log.callWithLogger(w, _read)

    # noinspection PyUnusedLocal
    def write(self, sock):
        self.notifier.setEnabled(False)
        if not self.watcher:
            return
        w = self.watcher

        def _write():
            try:
                data = w.doWrite()
                if data:
                    self.qt_reactor._disconnectSelectable(w, data, False)
                elif self.watcher:
                    self.notifier.setEnabled(True)
            except:
                log.err()
                self.qt_reactor._disconnectSelectable(w, sys.exc_info()[1], False)
            self.qt_reactor._iterate(None, fromqt=True)

        log.callWithLogger(w, _write)


# noinspection PyUnusedLocal
def msg_stub(msg_type, msg):
    pass


def msg_blast(msg_type, msg):
    log.msg("Qt says: ", msg_type, msg)


class QtReactor(posixbase.PosixReactorBase):
    implements(IReactorFDSet)

    def __init__(self):
        self._reads = {}
        self._writes = {}
        self._notifiers = {}
        self._timer = QtCore.QTimer()
        self._timer.setSingleShot(True)
        QtCore.QObject.connect(self._timer,
                               QtCore.SIGNAL("timeout()"),
                               self._qt_timeout)

        # noinspection PyArgumentList,PyArgumentList
        if QtCore.QCoreApplication.instance() is None:
            # Application Object has not been started yet
            self.qApp = QtCore.QCoreApplication([])
            self._ownApp = True
        else:
            self.qApp = QtCore.QCoreApplication.instance()
            self._ownApp = False
        self._blockApp = None

        QtCore.qInstallMsgHandler(msg_blast)

        super(QtReactor, self).__init__()

    def _add(self, xer, primary, select_rw):
        """
        Private method for adding a descriptor from the event loop.

        It takes care of adding it if  new or modifying it if already added
        for another state (read -> read/write for example).
        """
        if xer not in primary:
            primary[xer] = TwistedSocketNotifier(None, self, xer, select_rw)

    def addReader(self, reader):
        """
        Add a FileDescriptor for notification of data available to read.
        """
        self._add(reader, self._reads, QtCore.QSocketNotifier.Read)

    def addWriter(self, writer):
        """
        Add a FileDescriptor for notification of data available to write.
        """
        self._add(writer, self._writes, QtCore.QSocketNotifier.Write)

    # noinspection PyMethodMayBeStatic
    def _remove(self, xer, primary):
        """
        Private method for removing a descriptor from the event loop.

        It does the inverse job of _add, and also add a check in case of the fd
        has gone away.
        """
        if xer in primary:
            notifier = primary.pop(xer)
            notifier.shutdown()

    def removeReader(self, reader):
        """
        Remove a Selectable for notification of data available to read.
        """
        self._remove(reader, self._reads)

    def removeWriter(self, writer):
        """
        Remove a Selectable for notification of data available to write.
        """
        self._remove(writer, self._writes)

    def removeAll(self):
        """
        Remove all selectables, and return a list of them.
        """
        rv = self._removeAll(self._reads, self._writes)
        return rv

    def getReaders(self):
        return self._reads.keys()

    def getWriters(self):
        return self._writes.keys()

    def callLater(self, howlong, *args, **kargs):
        rval = super(QtReactor, self).callLater(howlong, *args, **kargs)
        self.invoke_reactor()
        return rval

    def invoke_reactor(self):
        self._timer.stop()
        self._timer.setInterval(0)
        self._timer.start()

    def _qt_timeout(self):
        self._iterate(None, True)

    def _iterate(self, delay, fromqt):
        """
        See twisted.internet.interfaces.IReactorCore.iterate.
        """
        self.runUntilCurrent()
        self._do_iteration(delay, fromqt)

    def doIteration(self, delay=None):
        self._do_iteration(delay, False)

    def _do_iteration(self, delay, fromqt):
        """
        This method is called by a Qt timer or by network activity
        on a file descriptor
        """
        if not self.running and self._blockApp:
            self._blockApp.quit()
        self._timer.stop()
        delay = max(delay, 1)
        if not fromqt:
            self.qApp.processEvents(QtCore.QEventLoop.AllEvents, delay * 1000)
        if self.timeout() is None:
            timeout = 0.1
        elif self.timeout() == 0:
            timeout = 0
        else:
            timeout = self.timeout()
        self._timer.setInterval(timeout * 1000)
        self._timer.start()

    def runReturn(self, installSignalHandlers=True):
        self.startRunning(installSignalHandlers=installSignalHandlers)
        self.invoke_reactor()

    def run(self, installSignalHandlers=True):
        if self._ownApp:
            self._blockApp = self.qApp
        else:
            self._blockApp = QtCore.QEventLoop()
        self.runReturn()
        self._blockApp.exec_()


class QtEventReactor(QtReactor):
    def __init__(self, *args, **kwargs):
        self._events = {}
        super(QtEventReactor, self).__init__()

    def addEvent(self, event, fd, action):
        """
        Add a new win32 event to the event loop.
        """
        self._events[event] = (fd, action)

    def removeEvent(self, event):
        """
        Remove an event.
        """
        if event in self._events:
            del self._events[event]

    def doEvents(self):
        handles = self._events.keys()
        if len(handles) > 0:
            val = None
            while val != WAIT_TIMEOUT:
                val = MsgWaitForMultipleObjects(handles, 0, 0,
                                                QS_ALLINPUT | QS_ALLEVENTS)
                if WAIT_OBJECT_0 <= val < WAIT_OBJECT_0 + len(handles):
                    event_id = handles[val - WAIT_OBJECT_0]
                    if event_id in self._events:
                        fd, action = self._events[event_id]
                        log.callWithLogger(fd, self._runAction, action, fd)
                elif val == WAIT_TIMEOUT:
                    pass
                else:
                    # print 'Got an unexpected return of %r' % val
                    return

    # noinspection PyBroadException
    def _runAction(self, action, fd):
        try:
            closed = getattr(fd, action)()
        except:
            closed = sys.exc_info()[1]
            log.deferr()

        if closed:
            self._disconnectSelectable(fd, closed, action == 'doRead')

    def timeout(self):
        t = super(QtEventReactor, self).timeout()
        return min(t, 0.01)

    def iterate(self, delay=None):
        """See twisted.internet.interfaces.IReactorCore.iterate.
        """
        self.runUntilCurrent()
        self.doEvents()
        self.doIteration(delay)


def posixinstall():
    """
    Install the Qt reactor.
    """
    p = QtReactor()
    from twisted.internet.main import installReactor

    installReactor(p)


def win32install():
    """
    Install the Qt reactor.
    """
    p = QtEventReactor()
    from twisted.internet.main import installReactor

    installReactor(p)


if runtime.platform.getType() == 'win32':
    # noinspection PyUnresolvedReferences
    from win32event import (WAIT_OBJECT_0, WAIT_TIMEOUT,
                            QS_ALLINPUT, QS_ALLEVENTS)

    install = win32install
else:
    install = posixinstall

if __name__ == '__main__':
    from twisted.internet import reactor

    print("ran with: ", reactor.__module__)

    trialpath = '/usr/local/bin/trial'
    trial = open(trialpath, 'r').read()

    import contextlib

    # noinspection PyProtectedMember
    @contextlib.contextmanager
    def redirect_argv(trials):
        sys._argv = sys.argv[:]
        sys.argv[:] = trials
        yield
        sys.argv[:] = sys._argv

    with redirect_argv([trialpath,
                        'twisted.test.test_ftp',
                        'twisted.test.test_internet']):
        exec trial
        print("ran with: ", reactor.__module__)

__all__ = ["install"]



from __future__ import print_function

__author__ = 'ght'

# Copyright (c) Twisted Matrix Laboratories.
# See LICENSE for details.

# ---------------------------------
# Client

from twisted.application import reactors

reactors.installReactor('qt4')

#reactors.installReactor('kqueue')

from twisted.python import log

import sys

log.startLogging(sys.stdout)

from twisted.internet.protocol import DatagramProtocol
from twisted.internet import reactor


# noinspection PyClassHasNoInit
class EchoClientDatagramProtocol(DatagramProtocol):
    long_string = "-" * 1000

    def __init__(self):
        self.sendflag = True

    def stop_sending(self):
        self.sendflag = False

    def startProtocol(self):
        #log.msg("client start protocol")
        self.transport.connect('127.0.0.1', self.useport)
        self.senddatagram()

    def senddatagram(self):
        #log.msg("client send datagram")
        datagram = self.long_string
        self.transport.write(datagram)

    def datagramReceived(self, datagram, host):
        #log.msg("client datagram received")
        #log.msg('Datagram received: ', repr(datagram))
        if self.sendflag:
            reactor.callLater(0.001, self.senddatagram)


def build_client(port):
    protocol = EchoClientDatagramProtocol()
    protocol.useport = port
    reactor.listenUDP(0, protocol)
    return protocol


#------------------------------------------
# Server


# noinspection PyClassHasNoInit
class EchoUDP(DatagramProtocol):
    def datagramReceived(self, datagram, address):
        #log.msg("server received: ", repr(datagram), repr(address))
        self.transport.write(datagram, address)

def trap_me():
    log.msg("trapped")

begin_port = 5010

def main():
    for port in xrange(begin_port, begin_port + 50):
        reactor.listenUDP(port, EchoUDP())
        protocol = build_client(port)
        reactor.callLater(12.0, protocol.stop_sending)

    reactor.callLater(10, trap_me)
    reactor.callLater(15, reactor.stop)
    reactor.run()


if __name__ == '__main__':
    main()


# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
# see LICENSE for details


from twisted.application.reactors import Reactor

qt4 = Reactor('qt4', 'qt4reactor', 'PyQt4 integration reactor')
pyqt4 = Reactor('pyqt4', 'qtreactor.pyqt4reactor', 'PyQt4 integration reactor')
pyside4 = Reactor('pyside4', 'qtreactor.pyside4reactor', 'PySide4 integration reactor')
qt4bad = Reactor('qt4bad', 'qt4reactor_bad', 'Qt4 broken reactor')



__author__ = 'ght'

import sys
from PyQt4 import QtGui

from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
import matplotlib.pyplot as plt

from twisted.application import reactors

reactors.installReactor('pyqt4')

import random

class Window(QtGui.QDialog):
    def __init__(self, parent=None):
        super(Window, self).__init__(parent)

        # a figure instance to plot on
        self.figure = plt.figure()

        # this is the Canvas Widget that displays the `figure`
        # it takes the `figure` instance as a parameter to __init__
        self.canvas = FigureCanvas(self.figure)

        # this is the Navigation widget
        # it takes the Canvas widget and a parent
        self.toolbar = NavigationToolbar(self.canvas, self)

        # Just some button connected to `plot` method
        self.button = QtGui.QPushButton('Plot')
        self.button.clicked.connect(self.plot)

        # set the layout
        layout = QtGui.QVBoxLayout()
        layout.addWidget(self.toolbar)
        layout.addWidget(self.canvas)
        layout.addWidget(self.button)
        self.setLayout(layout)

    def plot(self):
        ''' plot some random stuff '''
        # random data
        data = [random.random() for i in range(10)]

        # create an axis
        ax = self.figure.add_subplot(111)

        # discards the old graph
        ax.hold(False)

        # plot data
        ax.plot(data, '*-')

        # refresh canvas
        self.canvas.draw()

if __name__ == '__main__':
    app = QtGui.QApplication(sys.argv)

    main = Window()
    main.show()

    sys.exit(app.exec_())



from __future__ import print_function
from __future__ import absolute_import

import sys

from ghtTests.texboxtest import buildgui

form = buildgui()

from twisted.internet import reactor
from twisted.python import log

log.startLogging(sys.stdout)

def shutdown():
    log.msg("got shutdown")


reactor.addSystemEventTrigger('before','shutdown',shutdown)

log.msg('calling reactor.run()')
reactor.run()
log.msg('fell off the bottom?...')

